package com.qmth.distributed.print.business.activiti.custom.listener;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.qmth.distributed.print.business.entity.ExamTask;
import com.qmth.distributed.print.business.entity.TFFlowLog;
import com.qmth.distributed.print.business.enums.CustomFlowMultipleUserApproveTypeEnum;
import com.qmth.distributed.print.business.enums.MessageEnum;
import com.qmth.distributed.print.business.service.BasicMessageService;
import com.qmth.distributed.print.business.service.PrintCommonService;
import com.qmth.teachcloud.common.bean.params.ApproveUserResult;
import com.qmth.teachcloud.common.contant.SpringContextHolder;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.FlowApproveOperationEnum;
import com.qmth.teachcloud.common.enums.TFCustomTypeEnum;
import com.qmth.teachcloud.common.service.SysUserService;
import com.qmth.teachcloud.common.util.JacksonUtil;
import org.activiti.engine.HistoryService;
import org.activiti.engine.TaskService;
import org.activiti.engine.delegate.event.ActivitiEvent;
import org.activiti.engine.delegate.event.ActivitiEventListener;
import org.activiti.engine.delegate.event.ActivitiProcessStartedEvent;
import org.activiti.engine.history.HistoricVariableInstance;
import org.activiti.engine.task.Task;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.io.Serializable;
import java.util.*;

/**
 * @Description: 全局监听
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2022/2/8
 */
@Component
public class ProcessEventListener implements ActivitiEventListener, Serializable {
    private final static Logger log = LoggerFactory.getLogger(ProcessEventListener.class);

    @Override
    public void onEvent(ActivitiEvent event) {
        switch (event.getType()) {
            case PROCESS_STARTED:
                // 流程启动
//                log.info("流程启动_PROCESS_STARTED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
                BasicMessageService basicMessageService = SpringContextHolder.getBean(BasicMessageService.class);
                ActivitiProcessStartedEvent activitiProcessStartedEvent = (ActivitiProcessStartedEvent) event;
                Map<String, Object> map = activitiProcessStartedEvent.getVariables();
                Long objectId = (Long) map.get(SystemConstant.OBJECT_ID);
                TFCustomTypeEnum flowType = (TFCustomTypeEnum) map.get(SystemConstant.FLOW_TYPE);
                Object o = map.get(SystemConstant.OBJECT_DATA);
                Boolean flowSubmit = (Boolean) map.get(SystemConstant.FLOW_SUBMIT);
                //发送命题待审批短信
                if (Objects.nonNull(objectId) && Objects.nonNull(flowType) && Objects.nonNull(o) && Objects.nonNull(flowSubmit) && flowSubmit) {
                    if (flowType == TFCustomTypeEnum.ELECTRON_FLOW && o instanceof ExamTask) {
                        basicMessageService.sendNoticeTaskCreate((ExamTask) o);
                    }
                }
                break;
            case PROCESS_COMPLETED:
                // 流程结束
//                log.info("流程结束_PROCESS_COMPLETED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
                HistoryService historyService = SpringContextHolder.getBean(HistoryService.class);
                SysUserService sysUserService = SpringContextHolder.getBean(SysUserService.class);
                PrintCommonService printCommonService = SpringContextHolder.getBean(PrintCommonService.class);

                List<HistoricVariableInstance> historicVariableInstanceList = historyService.createHistoricVariableInstanceQuery().processInstanceId(event.getProcessInstanceId()).list();
                if (Objects.nonNull(historicVariableInstanceList) && historicVariableInstanceList.size() > 0) {
                    TFFlowLog tfFlowLog = null;
                    Object object = null;
                    for (HistoricVariableInstance h : historicVariableInstanceList) {
                        if (Objects.equals(h.getVariableName(), SystemConstant.APPROVE_TF_FLOW_LOG)) {
                            tfFlowLog = (TFFlowLog) h.getValue();
                            Optional.ofNullable(tfFlowLog).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程审批日志为空"));
                        } else if (Objects.equals(h.getVariableName(), SystemConstant.OBJECT_DATA)) {
                            object = h.getValue();
                            Optional.ofNullable(object).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("实体数据为空"));
                        }
                        if (Objects.nonNull(object) && Objects.nonNull(tfFlowLog)) {
                            break;
                        }
                    }
                    if (Objects.nonNull(tfFlowLog.getObjectTable())) {
                        if (Objects.equals(tfFlowLog.getObjectTable(), TFCustomTypeEnum.ELECTRON_FLOW.getTable())) {//如果是命题任务交卷
                            //取命题老师ID
                            SysUser sysUser = sysUserService.getById(((ExamTask) object).getUserId());
                            try {
                                printCommonService.checkData(((ExamTask) object).getSchoolId(), ((ExamTask) object).getCourseCode(), ((ExamTask) object).getPaperNumber(), sysUser);
                            } catch (IOException e) {
                                throw ExceptionResultEnum.ERROR.exception("生成pdf失败");
                            }
                        }
                    }
                }
                break;
            case TASK_COMPLETED:
                // 任务被完成了。它会在ENTITY_DELETE事件之前触发。当任务是流程一部分时，事件会在流程继续运行之前， 后续事件将是ACTIVITY_COMPLETE，对应着完成任务的节点。
//                log.info("流程任务完成_TASK_COMPLETED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
                TaskService taskService = SpringContextHolder.getBean(TaskService.class);
                sysUserService = SpringContextHolder.getBean(SysUserService.class);
                basicMessageService = SpringContextHolder.getBean(BasicMessageService.class);

                Task task = taskService.createTaskQuery().executionId(event.getExecutionId()).singleResult();
                if (Objects.nonNull(task)) {
                    TFFlowLog tfFlowLog = (TFFlowLog) taskService.getVariable(task.getId(), SystemConstant.APPROVE_TF_FLOW_LOG);
                    Optional.ofNullable(tfFlowLog).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程审批日志为空"));
                    o = taskService.getVariable(task.getId(), SystemConstant.OBJECT_DATA);
                    Optional.ofNullable(o).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("实体数据为空"));
                    if (tfFlowLog.getApproveOperation() != FlowApproveOperationEnum.SUBMIT
                            && tfFlowLog.getApproveOperation() != FlowApproveOperationEnum.END
                            && tfFlowLog.getApproveOperation() != FlowApproveOperationEnum.FINISH) {
                        if (Objects.nonNull(tfFlowLog.getPendApproveId())) {
                            String[] strs = tfFlowLog.getPendApproveId().split(",");
                            List<Long> userIds = new ArrayList<>();
                            for (String s : strs) {
                                userIds.add(SystemConstant.convertIdToLong(s.trim()));
                            }
                            List<SysUser> sysUsers = sysUserService.listByIds(userIds);
                            List<ApproveUserResult> sysUserList = new Gson().fromJson(JacksonUtil.parseJson(sysUsers), new TypeToken<List<ApproveUserResult>>() {
                            }.getType());

                            List<ApproveUserResult> nextApproverPeopleList = new ArrayList<>();
                            nextApproverPeopleList.addAll(sysUserList);
                            if (Objects.nonNull(tfFlowLog.getApproveUserApproveType()) && tfFlowLog.getApproveUserApproveType() == CustomFlowMultipleUserApproveTypeEnum.ORDER) {
                                nextApproverPeopleList.clear();
                                nextApproverPeopleList.add(sysUserList.get(0));
                            }

                            if (Objects.nonNull(nextApproverPeopleList) && (tfFlowLog.getApproveOperation() == FlowApproveOperationEnum.APPROVE
                                    || tfFlowLog.getApproveOperation() == FlowApproveOperationEnum.EXCHANGE)) {
                                //如果不是并行会签，则每个节点都发
                                if (Objects.nonNull(tfFlowLog.getApproveUserApproveType()) && tfFlowLog.getApproveUserApproveType() != CustomFlowMultipleUserApproveTypeEnum.ALL) {
                                    if (Objects.nonNull(o) && o instanceof ExamTask) {
                                        basicMessageService.sendNoticeTaskAuditFlow((ExamTask) o, nextApproverPeopleList, MessageEnum.NOTICE_OF_AUDIT_CREATED);
                                    }
                                } else {
                                    if (Objects.isNull(tfFlowLog.getApproveMultiSetup()) || tfFlowLog.getApproveMultiSetup().intValue() == 0) {
                                        if (Objects.nonNull(o) && o instanceof ExamTask) {
                                            basicMessageService.sendNoticeTaskAuditFlow((ExamTask) o, nextApproverPeopleList, MessageEnum.NOTICE_OF_AUDIT_CREATED);
                                        }
                                    }
                                }
                            } else if (Objects.nonNull(nextApproverPeopleList) && (tfFlowLog.getApproveOperation() == FlowApproveOperationEnum.REJECT
                                    || tfFlowLog.getApproveOperation() == FlowApproveOperationEnum.CANCEL)) {
                                if (Objects.nonNull(o) && o instanceof ExamTask) {
                                    basicMessageService.sendNoticeTaskAuditFlow((ExamTask) o, nextApproverPeopleList, MessageEnum.NOTICE_OF_AUDIT_REJECT);
                                }
                            }
                        }
                    }
                }
                break;
            default:
//                log.info("default->type:{},ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getType(), event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
                break;
        }
    }

    @Override
    public boolean isFailOnException() {
        return false;
    }
}
